@@ -1,4 +1,4 @@
-use crate::error::OkxError;
+use crate::{error::OkxError, websocket::types::messages::event::Change};
 
 use super::super::messages::event::{Event, ResponseKind};
 use exc::types::ticker::Ticker;
@@ -18,6 +18,13 @@ impl ServerFrame {
             Event::Response(ResponseKind::Unsubscribe { arg: _ } | ResponseKind::Error(_))
         )
     }
+
+    pub(crate) fn into_change(self) -> Option<Change> {
+        match self.inner {
+            Event::Response(_) => None,
+            Event::Change(change) => Some(change),
+        }
+    }
 }
 
 impl TryFrom<ServerFrame> for Vec<Result<Ticker, OkxError>> {
